# -*- coding: utf-8 -*-
#
# CRI Middleware SDK
# Copyright (c) CRI Middleware Co., Ltd.

import sys
import os.path
# CRI AtomCraft Python API
import cri.atomcraft.criatomcraft_api_lib as acconnect
import cri.atomcraft.debug as acdebug
import cri.atomcraft.project as acproject
import cri.atomcraft.preview as acpreview

# UE4 Python API
import unreal

# 破壊的
def pop_as_list(arrayobj):
    listobj = []
    listobj.append(arrayobj.pop())
    return listobj

def log_output(message):
    # AtomCraft と UE4 アウトプットログの両方にメッセージ出力
    acdebug.log(message)
    unreal.log(message)

def connect_atomcraft():
    # 通信ライブラリを初期化する
    val = acconnect.initialize()
    if val != 0:
        unreal.log("AtomCraft接続ライブラリの初期化に失敗")
        sys.exit()
    # 接続を開始する
    val = acconnect.connect("127.0.0.1", 9000)
    if val != 0:
        unreal.log("AtomCraftへの接続に失敗")
        acconnect.finalize()
        sys.exit()

def disconnect_atomcraft():    
    # 接続を終了し、ライブラリを終了する
    acconnect.disconnect()
    acconnect.finalize()

if __name__ == '__main__':
    unreal.log("AtomCraft スクリプトサーバに接続")
    connect_atomcraft()
    try:
        # 設定
        workunit_name = "WorkUnit_0"

        # 最後に選択した SoundAtomCue アセットを取得
        if unreal.EditorUtilityLibrary.get_selected_assets() == []:
            log_output("SoundAtomCue アセットを選択してください")

        ue4_last_selected_asset_on_editor = unreal.EditorUtilityLibrary.get_selected_assets().pop()
        unreal.log(ue4_last_selected_asset_on_editor.get_class().get_path_name())
        if not ue4_last_selected_asset_on_editor.get_class().get_path_name() == "/Script/CriWareRuntime.SoundAtomCue":
            raise ValueError("error!")

        # アセットのキュー名とキューシート名を取得
        cuesheet_name = ue4_last_selected_asset_on_editor.get_editor_property("CueSheet").get_editor_property("CueSheetName")
        cue_name = ue4_last_selected_asset_on_editor.get_editor_property("CueName")

        # ワークユニットを取得する
        result = acproject.get_workunit(workunit_name)
        if not result["succeed"]:
            log_output("ワークユニットの取得に失敗")
            raise ValueError("error!")
        workunit = result["data"]

        # キューシートを取得する
        result = acproject.find_object(workunit, "CueSheet", cuesheet_name)
        if not result["succeed"]:
            log_output("キューシートの取得に失敗")
            raise ValueError("error!")
        cuesheet = result["data"]

        # キューを取得する
        result = acproject.find_object(workunit, "Cue", cue_name)
        if not result["succeed"]:
            log_output("キューの取得に失敗")
            raise ValueError("error!")
        cue = result["data"]

        # AtomCraft 側でキューを選択
        acproject.select_object(cue)

    except:
        unreal.log("スクリプトの例外が発生しました")
        #unreal.log(traceback.format_exc()) # デバッグ用。トレースバックを表示した場合、例外発生じにはAtomCraftサーバを終了しなければPythonスクリプトを再実行できない

    unreal.log("AtomCraft スクリプトサーバから切断")
    disconnect_atomcraft()
